/* Copyright (c) 2012 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRANTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 */

#include <stdint.h>
#include <string.h>

#include "app_util.h"
#include "nordic_common.h"
#include "ble_srv_common.h"

#include "ble_cts_date_time.h"
#include "ble_cts_alford.h"

#include "m_debugmsg.h"
#include "m_rtcc.h"


/**@brief Connect event handler.
 *
 * @param[in]   p_cts       Alford Current Time Service structure.
 * @param[in]   p_ble_evt   Event received from the BLE stack.
 */
static void on_connect(ble_cts_t * p_cts, ble_evt_t * p_ble_evt)
{
    p_cts->conn_handle = p_ble_evt->evt.gap_evt.conn_handle;
}


/**@brief Disconnect event handler.
 *
 * @param[in]   p_cts       Alford Current Time Service structure.
 * @param[in]   p_ble_evt   Event received from the BLE stack.
 */
static void on_disconnect(ble_cts_t * p_cts, ble_evt_t * p_ble_evt)
{
    UNUSED_PARAMETER(p_ble_evt);
    p_cts->conn_handle = BLE_CONN_HANDLE_INVALID;
}


/**@brief Write event handler.
 *
 * @param[in]   p_cts       Alford Current Time Service structure.
 * @param[in]   p_ble_evt   Event received from the BLE stack.
 */
static void on_write(ble_cts_t * p_cts, ble_evt_t * p_ble_evt)
{
    ble_cts_date_time_t alarm_time, time_stamp;
    ble_cts_evt_t cts_evt_type;
    ble_gatts_evt_write_t * p_evt_write = &p_ble_evt->evt.gatts_evt.params.write;

    if (sizeof(ble_cts_date_time_t) != p_evt_write->len)
    {
        return;
    }

    if (p_evt_write->handle == p_cts->alarm_handles.value_handle)
    {
        dbmsg_ftrace(DBM_DATA, "Alarm Clock Update len: %d\r\n", p_evt_write->len);
        ble_cts_date_time_decode(&alarm_time, p_evt_write->data);
        m_rtccIoctl(&sRTCC_CURRENT, RTCC_W_ALARMTIME, &alarm_time);
        cts_evt_type.cts_evt_type = BLE_CTS_EVT_ALARM_CLOCK_CONFIG;

        if(NULL != p_cts->evt_handler)
        {
            p_cts->evt_handler(p_cts, &cts_evt_type, p_evt_write->data, p_evt_write->len);
        }
    }
    else if (p_evt_write->handle == p_cts->current_time_handles.value_handle)
    {
        dbmsg_ftrace(DBM_DATA, "Current Time Update lend: %d\r\n", p_evt_write->len);
        ble_cts_date_time_decode(&time_stamp, p_evt_write->data);
        m_rtccIoctl(&sRTCC_CURRENT, RTCC_W_DATATIME, &time_stamp);
        cts_evt_type.cts_evt_type = BLE_CTS_EVT_CURRENT_TIME_CONFIG;
				//resive
//         m_lcd_showPeriod(time_stamp);

        if(NULL != p_cts->evt_handler)
        {
            p_cts->evt_handler(p_cts, &cts_evt_type, p_evt_write->data, p_evt_write->len);
        }
    }
}


void ble_cts_on_ble_evt(ble_cts_t * p_cts, ble_evt_t * p_ble_evt)
{
    dbmsg_ftrace(DBM_BLE, "ble_cts_alford.c - ble_cts_on_ble_evt event: %02d\r\n", p_ble_evt->header.evt_id);

    switch (p_ble_evt->header.evt_id)
    {
    case BLE_GAP_EVT_CONNECTED:
        on_connect(p_cts, p_ble_evt);
        break;

    case BLE_GAP_EVT_DISCONNECTED:
				dbmsg_ftrace(DBM_BOND, "cts services disconnect\r\n");
        on_disconnect(p_cts, p_ble_evt);
        break;

    case BLE_GATTS_EVT_WRITE:
        on_write(p_cts, p_ble_evt);
        break;

    default:
        break;
    }
}

/**@brief Add RX characteristic.
 *
 * @param[in]   p_cts        Alford Current Time Service structure.
 * @param[in]   p_cts_init   Information needed to initialize the service.
 *
 * @return      NRF_SUCCESS on success, otherwise an error code.
 */
static uint32_t current_time_char_add(ble_cts_t * p_cts, const ble_cts_init_t * p_cts_init)
{
    ble_gatts_char_md_t char_md;
    ble_gatts_attr_md_t cccd_md;
    ble_gatts_attr_t    attr_char_value;
    ble_uuid_t          ble_uuid;
    ble_gatts_attr_md_t attr_md;
    // Add Battery Level characteristic
    memset(&cccd_md, 0, sizeof(cccd_md));
    memset(&char_md, 0, sizeof(char_md));
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.write_perm);
//     BLE_UUID_BLE_ASSIGN(ble_uuid, BLE_UUID_CURRENT_TIME_CHAR);
    memset(&attr_md, 0, sizeof(attr_md));
    BLE_GAP_CONN_SEC_MODE_SET_ENC_NO_MITM(&attr_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_ENC_NO_MITM(&attr_md.write_perm);
    memset(&attr_char_value, 0, sizeof(attr_char_value));
    cccd_md.vloc              = BLE_GATTS_VLOC_STACK;
    char_md.char_props.write  = 1;
    char_md.char_props.read   = 1;
    char_md.p_char_user_desc  = NULL;
    char_md.p_char_pf         = NULL;
    char_md.p_user_desc_md    = NULL;
    char_md.p_cccd_md         = &cccd_md;
    char_md.p_sccd_md         = NULL;
    ble_uuid.type          = p_cts->uuid_type;
    ble_uuid.uuid          = BLE_UUID_CTS_CURRENT_CHARACTERISTIC;
    attr_md.vloc              = BLE_GATTS_VLOC_STACK;
    attr_md.rd_auth           = 0;
    attr_md.wr_auth           = 0;
    attr_md.vlen              = 1;
    attr_char_value.p_uuid    = &ble_uuid;
    attr_char_value.p_attr_md = &attr_md;
    attr_char_value.init_len  = sizeof(uint8_t);
    attr_char_value.init_offs = 0;
    attr_char_value.max_len   = 20;
    return sd_ble_gatts_characteristic_add(p_cts->service_handle, &char_md,
                                           &attr_char_value,
                                           &p_cts->current_time_handles);
}

/**@brief Add TX characteristic.
 *
 * @param[in]   p_cts        Alford Current Time Service structure.
 * @param[in]   p_cts_init   Information needed to initialize the service.
 *
 * @return      NRF_SUCCESS on success, otherwise an error code.
 */
static uint32_t alarm_clock_char_add(ble_cts_t * p_cts, const ble_cts_init_t * p_cts_init)
{
    ble_gatts_char_md_t char_md;
    ble_gatts_attr_t    attr_char_value;
    ble_uuid_t          ble_uuid;
    ble_gatts_attr_md_t attr_md;
    ble_gatts_attr_md_t cccd_md;
	
    memset(&cccd_md, 0, sizeof(cccd_md));
    memset(&char_md, 0, sizeof(char_md));
    memset(&attr_md, 0, sizeof(attr_md));
	
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.write_perm);
	
    BLE_GAP_CONN_SEC_MODE_SET_ENC_NO_MITM(&attr_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_ENC_NO_MITM(&attr_md.write_perm);
    memset(&attr_char_value, 0, sizeof(attr_char_value));
    cccd_md.vloc              = BLE_GATTS_VLOC_STACK;
    char_md.char_props.write  = 1;
    char_md.char_props.read   = 1;
    char_md.p_char_user_desc  = NULL;
    char_md.p_char_pf         = NULL;
    char_md.p_user_desc_md    = NULL;
    char_md.p_cccd_md         = &cccd_md;
    char_md.p_sccd_md         = NULL;
    ble_uuid.type             = p_cts->uuid_type;
    ble_uuid.uuid             = BLE_UUID_CTS_ALARM_CHARACTERISTIC;
    attr_md.vloc              = BLE_GATTS_VLOC_STACK;
    attr_md.rd_auth           = 0;
    attr_md.wr_auth           = 0;
    attr_md.vlen              = 1;
    attr_char_value.p_uuid    = &ble_uuid;
    attr_char_value.p_attr_md = &attr_md;
    attr_char_value.init_len  = sizeof(uint8_t);
    attr_char_value.init_offs = 0;
    attr_char_value.max_len   = 20;
    return sd_ble_gatts_characteristic_add(p_cts->service_handle, &char_md,
                                           &attr_char_value,
                                           &p_cts->alarm_handles);
}

uint32_t ble_cst_init(ble_cts_t * p_cts, const ble_cts_init_t * p_cts_init)
{
    uint32_t   err_code;
    ble_uuid_t ble_uuid;
    // Initialize service structure
//     ble_uuid128_t cts_base_uuid     = {0x9E, 0xCA, 0xDC, 0x24, 0x0E, 0xE5, 0xA9, 0xE0, 0x93, 0xF3, 0xA3, 0xB5, 0x00, 0x00, 0x40, 0x6E};
    p_cts->conn_handle              = BLE_CONN_HANDLE_INVALID;
    p_cts->evt_handler              = p_cts_init->evt_handler;
		p_cts->uuid_type 								= p_cts_init->uuid_type;
    p_cts->is_notification_enabled  = false;
//     err_code                        = sd_ble_uuid_vs_add(&cts_base_uuid, &p_cts->uuid_type);

//     if (err_code != NRF_SUCCESS)
//     {
//         return err_code;
//     }

    // Add service
    ble_uuid.type = p_cts_init->uuid_type;
    ble_uuid.uuid = BLE_UUID_CTS_ALFORD_SERVICE;
    err_code      = sd_ble_gatts_service_add(BLE_GATTS_SRVC_TYPE_PRIMARY, &ble_uuid, &p_cts->service_handle);

    if (err_code != NRF_SUCCESS)
    {
        return err_code;
    }

    err_code = alarm_clock_char_add(p_cts, p_cts_init);

    if (err_code != NRF_SUCCESS)
    {
        return err_code;
    }

    err_code = current_time_char_add(p_cts, p_cts_init);

    if (err_code != NRF_SUCCESS)
    {
        return err_code;
    }

    return NRF_SUCCESS;
}
